PROGRAM sistema_fichero
IMPLICIT NONE

INTEGER, PARAMETER:: n=500, fdat=11, fsal=12, nprob=10, nmetodos=2
DOUBLE PRECISION a(n,n), b(n), x(n), xsol(n)
INTEGER ipiv(n)
DOUBLE PRECISION err, t1, t2
DOUBLE PRECISION error(nprob,nmetodos), tiempo(nprob,nmetodos)
INTEGER info
INTEGER i, j, tipo
CHARACTER(LEN=30) pc(2)
pc(1) = 'I7-1065G7, 1.30GHz, 16Gb RAM'
pc(2) = 'Quad Q2800, 2.33GHz, 4Gb RAM'

OPEN (fdat, FILE='copia-sistema.dat')

OPEN (fsal, FILE='Sistema1.sal', POSITION='append')    
WRITE (*,'(A,2(/5X,I1,'': '',A)/10X,A)',ADVANCE='NO') 'Tipo de PC = ', &
      (i,pc(i),i=1,2), 'Opcion = '
READ*, tipo      
WRITE (fsal,'(/2A/)') 'Tipo de PC: ', pc(tipo)      
WRITE (fsal,'(A,I0)') 'n = ', n

error = 0 ; tiempo = 0

DO i = 1, nprob
  PRINT '(A,I2)', 'Sistema ', i
  
  ! Generacion del sistema

  REWIND (fdat)
  CALL RANDOM_NUMBER (a)
  a = 0.1d0 + 0.8d0*a
  CALL RANDOM_NUMBER (xsol)
  xsol = 0.1d0 + 0.8d0*xsol   ! verdadera solucion
  b = MATMUL(a,xsol)
  DO j = 1, n
    WRITE (fdat,'(F25.15,10000F20.16)') b(j), a(j,1:n)
  ENDDO
  
  ! Metodo 1: NR

  REWIND (fdat)
  CALL cpu_time (t1)
  DO j = 1, n
    READ (fdat,*) b(j), a(j,1:n)
  ENDDO
  
  CALL lusol (n, a, n, b, info)   ! en b devuelve la solucion
  error(i,1) = SUM(ABS(b-xsol))
  CALL cpu_time (t2)
  tiempo(i,1) = t2 - t1

 ! Metodo 2: LAPACK

  REWIND (fdat)
  CALL cpu_time (t1)
  DO j = 1, n
    READ (fdat,*) b(j), a(j,1:n)
  ENDDO

  CALL dgetrf (n, n, a, n, ipiv, info)   ! a = LU, diag(L) = 1
  CALL dgetrs ('N', n, 1, a, n, ipiv, b, n, info)
  error(i,2) = SUM(ABS(b-xsol))
  CALL cpu_time (t2)
  tiempo(i,2) = t2 - t1
ENDDO

! Escritura de resultados

WRITE (fsal,'(4A)') '       error NR', '   error LAPACK', &
                   '      tiempo NR', '  tiempo LAPACK'
DO i = 1, nprob
  WRITE (fsal,'(2E15.5,2F15.3)') error(i,:), tiempo(i,:)
ENDDO
WRITE (fsal,'(/A/2E15.5,2F15.3)') 'Resultados medios', &
   SUM(error(:,1))/nprob, SUM(error(:,2))/nprob, &
   SUM(tiempo(:,1))/nprob, SUM(tiempo(:,2))/nprob

WRITE (*,'(/A/4A)') 'Resultados medios',    &
      '       error NR', '   error LAPACK', &
      '      tiempo NR', '  tiempo LAPACK'
WRITE (*,'(2E15.5,2F15.3)') &
   SUM(error(:,1))/nprob, SUM(error(:,2))/nprob, &
   SUM(tiempo(:,1))/nprob, SUM(tiempo(:,2))/nprob

ENDPROGRAM sistema_fichero

! ======================================================================
! RESOLUCION DEL SISTEMA Ax = b. Adaptacion de Numerical Recipes ludcmp
! ----------------------------------------------------------------------

! INTEGER:: ia <=> dimension de filas de a
! DOUBLE PRECISION:: a(ia,*) <=> matriz del sistema
! INTEGER:: n <=> numero de ecuaciones
! DOUBLE PRECISION:: b(*) <=> vector del lado derecho, en salida b es 
!                              la solucion
! INTEGER:: ierlu <=> indicador de error
!           ierlu = 0 : solucion correcta
!           ierlu = -1: pivote nulo; no hay solucion o no es unica
! ----------------------------------------------------------------------

SUBROUTINE lusol (ia, a, n, b, ierlu)
IMPLICIT NONE

! Argumentos

INTEGER, INTENT(IN):: ia, n
INTEGER, INTENT(OUT):: ierlu
DOUBLE PRECISION, INTENT(INOUT):: a(ia,*), b(*)

! Variables locales

INTEGER i, imax, j, k, ii, ll
INTEGER ind(n)
DOUBLE PRECISION vv(n)
DOUBLE PRECISION aamax, dum, suma
DOUBLE PRECISION, PARAMETER:: eps=1.d-20

! ----------------------------
! Primera sentencia ejecutable

ierlu = 0
DO i = 1, n
  aamax = MAXVAL(ABS(a(i,1:n)))
  IF (aamax < eps) THEN
    ierlu = -1 ; RETURN
  ENDIF
  vv(i) = 1.d0 / aamax
ENDDO

DO j = 1, n

  DO i = 1, j-1
    suma = a(i,j)
    DO k = 1, i-1 ; suma = suma - a(i,k)*a(k,j) ; ENDDO
    a(i,j) = suma
  ENDDO

  aamax = 0.d0
  DO i = j, n
    suma = a(i,j)
    DO k = 1, j-1 ; suma = suma - a(i,k)*a(k,j) ; ENDDO
    a(i,j) = suma
    dum = vv(i) * ABS(suma)
    IF (dum >= aamax) THEN
      imax = i ; aamax = dum
    ENDIF
  ENDDO

  IF (j /= imax) THEN
    DO k = 1, n
      dum = a(imax,k) ; a(imax,k) = a(j,k) ; a(j,k) = dum
    ENDDO
    vv(imax) = vv(j)
  ENDIF

  ind(j) = imax
  IF (ABS(a(j,j)) < eps) a(j,j) = eps
  IF (j /= n) a(j+1:n,j) = a(j+1:n,j) / a(j,j)

ENDDO

ii = 0
DO i = 1, n
  ll = ind(i) ; suma = b(ll) ; b(ll) = b(i)
  IF (ii /= 0) THEN
    DO j = ii, i-1 ; suma = suma - a(i,j)*b(j) ; ENDDO
  ELSEIF (ABS(suma) >= eps) THEN
    ii = i
  ENDIF
  b(i) = suma
ENDDO

DO i = n, 1, -1
  suma = b(i)
  DO j = i+1, n ; suma = suma - a(i,j)*b(j) ; ENDDO
  b(i) = suma / a(i,i)
ENDDO

ENDSUBROUTINE lusol
